import { Entity, EntityNames } from ".";
import { BoundingBox, Colliding, Grid, Sprite } from "../components";
import { IMAGES, SPRITE_SPECS, SpriteSpec, Sprites } from "../config";
import { Coord2D } from "../interfaces";

export class Wall extends Entity {
  private static spriteSpec: SpriteSpec = SPRITE_SPECS.get(
    Sprites.WALL,
  ) as SpriteSpec;

  constructor(gridPosition: Coord2D) {
    super(EntityNames.Wall);

    this.addComponent(new Grid(gridPosition));

    this.addComponent(new Colliding());

    this.addComponent(
      new BoundingBox(
        {
          x: 0,
          y: 0,
        },
        {
          width: Wall.spriteSpec.width,
          height: Wall.spriteSpec.height,
        },
        0,
      ),
    );

    this.addComponent(
      new Sprite(
        IMAGES.get(Wall.spriteSpec.sheet)!,
        { x: 0, y: 0 },
        {
          width: Wall.spriteSpec.width,
          height: Wall.spriteSpec.height,
        },
        Wall.spriteSpec.msPerFrame,
        Wall.spriteSpec.frames,
      ),
    );
  }
}
